'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.assign = assign;
exports.interpolate = interpolate;
exports.toCamelCase = toCamelCase;
/**
 * @return {*}
 * @private
 */
function assign() {
    return Array.prototype.reduce.call(arguments, function (target, source) {
        return Object.keys(Object(source)).reduce(function (target, key) {
            target[key] = source[key];
            return target;
        }, target);
    });
}

/**
 *
 * @param value
 * @return {XML|string}
 * @private
 */
function stringify(value) {
    return value.replace(/'/gim, '\\\'').replace(/\n/gi, '\\n');
}

/**
 *
 * @param value
 * @param options
 * @return {string}
 * @private
 */
function interpolate(value, options) {
    var result = void 0;
    var statements = [];
    var lastIndex = 0;
    while ((result = options.interpolation.exec(value)) !== null) {
        var full = result[0];
        var group = result[1];
        var index = result.index;
        var before = value.substring(lastIndex, index);
        if (before) {
            statements.push('\'' + stringify(before) + '\'');
        }
        if (group.trim()) {
            statements.push('(' + group.trim() + ' === undefined || ' + group.trim() + ' === null ? \'\' : ' + group.trim() + ')');
        }
        lastIndex = index + full.length;
    }
    var after = value.substring(lastIndex, value.length);
    if (after) {
        statements.push('\'' + stringify(after) + '\'');
    }
    return statements.join(' + ');
}

/**
 * Converts string to camel case.
 * @param {!string} string the string to convert
 * @return {string} the camel cased string
 * @private
 */
function toCamelCase(string) {
    return string.toLowerCase().split('-').map(function (part, index) {
        return index ? part.charAt(0).toUpperCase() + part.slice(1) : part;
    }).join('');
}